function [r, a, q] = sensiblerange(a, neqstds)
% [R A Q] = SENSIBLERANGE(A, [N_EQ_STDS])
% 
% Calculates sensible ranges for the display of the distributions
% in the columns of A, based on a certain "number of equivalent
% standard deviations" (N_EQ_STDS) from the median.
% 
% For values above the median, an "equivalent standard deviation"
% (EQSTD) is the 0.841 quantile - the median, and for values below
% it is the median - the 0.159 quantile.  Thus, for perfectly normal
% distributions, the EQSTD /is/ equal to the standard deviation in
% both cases.
% 
% N_EQ_STDS can be a two-element vector, dictating the number of
% EQSTDs. to go [below_the_median, above_the_median].
% 
% Alternatively, N_EQ_STDS  can be a scalar value. In this case,
% interpretations vary depending on the data:
% 	If all values >= 0, then N_EQ_STDS refers to the number of
% 	EQSTDS to go above the median, and values below the median
% 	are not limited.
% 
% 	If all values <= 0, then N_EQ_STDS refers to the number of
% 	EQSTDS to go below the median, and values above the median
% 	are not limited.
% 
% 	If there are some positive and some negative values, then
% 	the scalar value N_EQ_STDS applies above and below the
% 	median
% 
% If N_EQ_STDS is omitted, a value is chosen such that the
% sensible range would contain 95% of the points (either one-tailed
% or two-tailed, as above) if the distributions were perfectly
% normal
% 	
% The sensible ranges are returned in the columns of R. Optionally,
% A is returned, with each column clipped to the appropriate range.
% Output argument Q contains the quantiles [0.159; 0.5; 0.841] of
% each distribution.
% 
% See also SCALESCORES

% Part of PFCMP version 2.5.41, an extension to the PSIGNIFIT toolbox for MATLAB versions 5 and up.
% The license and NO WARRANTY statement for PSIGNIFIT also applies to this release - see psych_legal.m
% Copyright (c) J.Hill 2002.
% mailto:psignifit@bootstrap-software.org
% http://www.bootstrap-software.org/psignifit/

if nargin < 2, neqstds = []; end

defaultEquivCoverage = 0.95;
defaultOneSided = phiinv(defaultEquivCoverage);
defaultTwoSided = c2z(defaultEquivCoverage);

if prod(size(neqstds)) < 2
	switch sign(sum(sign([min(a(:)) max(a(:))])))
	case -1 % all data <= 0
		if isempty(neqstds), neqstds = defaultOneSided; end
		neqstds = [neqstds inf];
	case 0
		if isempty(neqstds), neqstds = defaultTwoSided; end
		neqstds = [1 1] * neqstds;
	case +1 % all data >= 0
		if isempty(neqstds), neqstds = defaultOneSided; end
		neqstds = [inf neqstds];
	end
end

if prod(size(neqstds)) ~= 2, error('N_EQ_STDS should have 1 or 2 elements'), end
neqstds = abs(neqstds);

q = quantile(a, phi([-1;0;1]));
r = diff(q);
r(1, :) = q(2, :) - neqstds(1) * r(1, :);
r(2, :) = q(2, :) + neqstds(2) * r(2, :);

rpresent = [min(a, [], 1); max(a, [], 1)];
bad = (isinf(r) | isnan(r));
r(bad) = rpresent(bad);


if nargout >= 2, a = clip(a, r(1, :), r(2, :)); end
